/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Basic application to run a generic Scale Stack instance.
 */

#include <exception>
#include <iostream>

#include <scalestack/kernel/core.h>

using namespace std;

int main(int argc, const char* argv[])
{
  scalestack::kernel::core core;

  try
  {
    /* Use libevent as the default event service. */
    core.add_module_option_value("event::service", "", "event::libevent");

    /* The first argv entry is the execution filename, so skip it. */
    core.parse_arguments(argc - 1, argv + 1);

    core.set_signal_handlers();
    core.run();
  }
  catch (exception& e)
  {
    cout << e.what() << endl;
    return 1;
  }

  return 0;
}
