/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Database Proxy Client Service Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_CLIENT_SERVICE_H
#define SCALESTACK_DATABASEPROXY_CLIENT_SERVICE_H

#include <vector>

#include <scalestack/network/ip/tcp/connection_service.h>
#include <scalestack/network/stream_service.h>

namespace scalestack
{
namespace database_proxy
{

class client;
class connection;

class client_service: public network::stream_service
{
public:

  client_service(kernel::module& module);

  ~client_service();

  /**
   * See network::stream_service::add_stream().
   */
  network::stream* add_stream(void);

  /**
   * See network::stream_service::remove_stream().
   */
  void remove_stream(network::stream* stream);

  /**
   * Get a client connection to proxy to.
   */
  client* get_client(connection* connection, bool need_handshake);

  /**
   * Yield a client to other pending transactions.
   */
  void yield_client(client* client);

  /**
   * Erase a connection from the queue.
   */
  void remove_from_queue(connection* connection);

private:

  /**
   * Don't allow copying of objects.
   */
  client_service(const client_service&);

  /**
   * Don't allow assignment of objects.
   */
  client_service& operator=(const client_service&);

  size_t _max_clients;
  size_t _current_clients;
  pthread_mutex_t _queue_mutex;
  std::vector<connection*> _waiting_connections;
  std::vector<client*> _free_clients;
  network::ip::tcp::connection_service _connection_service;
};

} /* namespace database_proxy */
} /* namespace scalestack */

#endif /* SCALESTACK_DATABASEPROXY_CLIENT_SERVICE_H */
