/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Database Proxy Connection Declarations
 */

#ifndef SCALESTACK_DATABASEPROXY_CONNECTION_H
#define SCALESTACK_DATABASEPROXY_CONNECTION_H

#include <scalestack/network/stream.h>

namespace scalestack
{
namespace database_proxy
{

class client;
class service;

class connection: public network::stream
{
public:

  connection(service& creator);

  /**
   * See network::stream::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the client from the read buffer.
   */
  void continue_write(void);

  /**
   * See network::stream::flush_write().
   */
  void flush_write(void);

  /**
   * Start shutdown process.
   */
  void error(void);

  /**
   * Process shutdown requests, detaching from client.
   */
  void run(void);

  /**
   * Notify connection it can yielf the client now.
   */
  void yield(void);

private:

  /**
   * Don't allow copying of objects.
   */
  connection(const connection&);

  /**
   * Don't allow assignment of objects.
   */
  connection& operator=(const connection&);

  /**
   * Write data to the client. This is used by both the read() and
   * continue_write() methods.
   */
  size_t _client_write(void);

  enum
  {
    CLIENT_HANDSHAKE_PACKET,
    CLIENT_HANDSHAKE_FLUSH,
    COMMAND_PACKET,
    COMMAND_FLUSH
  } _state;
  bool _need_handshake;
  bool _need_yield;
  bool _in_transaction;
  bool _shutdown;
  size_t _size;
  size_t _packet_size;
  uint8_t* _buffer;
  client* _client;
};

} /* namespace database_proxy */
} /* namespace scalestack */

#endif /* SCALESTACK_DATABASEPROXY_CONNECTION_H */
