/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Database Proxy Service Definitions
 */

#include "config.h"

#include <scalestack/database_proxy/connection.h>
#include <scalestack/database_proxy/service.h>

namespace scalestack
{
namespace database_proxy
{

/*
 * Public functions.
 */

service::service(kernel::module& module):
  network::stream_service(module),
  _connection_service(module, *this),
  _listener_service(module, _connection_service),
  _client_service(module)
{
  _listener_service.add_listeners();
}

service::~service()
{
}

network::stream* service::add_stream(void)
{
  return new connection(*this);
}

void service::remove_stream(network::stream* stream)
{
  delete stream;
}

client* service::get_client(connection* connection, bool need_handshake)
{
  return _client_service.get_client(connection, need_handshake);
}

void service::yield_client(client* client)
{
  _client_service.yield_client(client);
}

void service::remove_from_queue(connection* connection)
{
  _client_service.remove_from_queue(connection);
}

} /* namespace database_proxy */
} /* namespace scalestack */
