/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Datagram Declarations
 */

#ifndef SCALESTACK_ECHO_FLOOD_DATAGRAM_H
#define SCALESTACK_ECHO_FLOOD_DATAGRAM_H

#include <scalestack/kernel/macros.h>
#include <scalestack/network/datagram.h>

namespace scalestack
{
namespace kernel
{

class module;

} /* namespace kernel */

namespace echo
{
namespace flood
{

/**
 * Class for handling datagram requests.
 */
class SCALESTACK_API datagram: public network::datagram
{
public:

  datagram(network::datagram_service& creator);

  ~datagram();

  /**
   * See network::datagram::ready().
   */
  void ready(void);

  /**
   * See network::datagram::receive().
   */
  size_t receive(uint8_t* buffer,
                 size_t size,
                 struct sockaddr& peer,
                 socklen_t peer_size);

  /**
   * See network::datagram::flush_send().
   */
  void flush_send(void);

  /**
   * See network::datagram::error().
   */
  void error(void);

private:

  /**
   * Don't allow copying of objects.
   */
  datagram(const datagram&);

  /**
   * Don't allow assignment of objects.
   */
  datagram& operator=(const datagram&);

  size_t _iterations;
  size_t _packet_size;
  size_t _read_offset;
  size_t _write_offset;
};

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_FLOOD_DATAGRAM_H */
