/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Datagram Service Definitions
 */

#include "config.h"

#include <scalestack/echo/flood/datagram.h>
#include <scalestack/echo/flood/datagram_service.h>

namespace scalestack
{
namespace echo
{
namespace flood
{

/*
 * Public functions.
 */

datagram_service::datagram_service(kernel::module& module):
  network::datagram_service(module),
  _count(0)
{
}

datagram_service::~datagram_service()
{
}

network::datagram* datagram_service::add_datagram(void)
{
  _count++;
  return new datagram(*this);
}

void datagram_service::remove_datagram(network::datagram* datagram)
{
  delete datagram;

  _count--;
  if (_count == 0)
    _module.get_core().shutdown();
}

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */
