/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Datagram Service Declarations
 */

#ifndef SCALESTACK_ECHO_FLOOD_DATAGRAM_SERVICE_H
#define SCALESTACK_ECHO_FLOOD_DATAGRAM_SERVICE_H

#include <scalestack/network/datagram_service.h>

namespace scalestack
{
namespace echo
{
namespace flood
{

class datagram_service: public network::datagram_service
{
public:

  datagram_service(kernel::module& module);

  ~datagram_service();

  /**
   * See network::datagram_service::add_datagram().
   */
  network::datagram* add_datagram(void);

  /**
   * See network::datagram_service::remove_datagram().
   */
  void remove_datagram(network::datagram* datagram);

private:

  /**
   * Don't allow copying of objects.
   */
  datagram_service(const datagram_service&);

  /**
   * Don't allow assignment of objects.
   */
  datagram_service& operator=(const datagram_service&);

  size_t _count;
};

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_FLOOD_DATAGRAM_SERVICE_H */
