/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Module Definitions
 */

#include "config.h"

#include <cstdlib>
#include <memory>

#include <scalestack/echo/flood/datagram_service.h>
#include <scalestack/echo/flood/flood.h>
#include <scalestack/echo/flood/stream_service.h>
#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace std;
using namespace scalestack;

uint8_t echo::flood::random_buffer[echo::flood::random_buffer_size];

static void options(kernel::module& module)
{
  module.add_option("iterations",
                    _("Number of iterations to run, 0 means forever."),
                    "COUNT",
                    "1");
  module.add_option("packet_size",
                    _("Size of datagram packets to send."),
                    "SIZE",
                    "4096");
  module.add_option("reconnect",
                    _("Whether to reconnect between iterations for streams."),
                    "BOOLEAN",
                    "False");
}

static void start(kernel::module& module)
{
  for (size_t index = 0; index < echo::flood::random_buffer_size; index++)
    echo::flood::random_buffer[index] = static_cast<uint8_t>(rand());

  auto_ptr<echo::flood::datagram_service> datagram_service(new echo::flood::datagram_service(module));

  module.get_core().add_service("echo::flood::datagram",
                                datagram_service.get());

  auto_ptr<echo::flood::stream_service> stream_service(new echo::flood::stream_service(module));

  module.get_core().add_service("echo::flood::stream", stream_service.get());

  datagram_service.release();
  stream_service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::flood::datagram");
  delete module.get_core().remove_service("echo::flood::stream");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
