/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Local Stream Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/network/stream_service.h>
#include <scalestack/network/local/stream/connection_service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  /* Replace this option with default from network module */
  module.add_option("local_socket", "Local socket to connect to.", "SOCKET", "/tmp/echoserver.socket");
  //network::local::stream::connection_service::options(module, "/tmp/echoserver.socket");
  module.add_option("count", "Number of concurrent connections to open.", "COUNT", "1");
}

static void start(kernel::module& module)
{
  auto_ptr<network::local::stream::connection_service> service(new network::local::stream::connection_service(module, static_cast<network::stream_service&>(module.get_core().get_service("echo::flood::stream"))));
  module.get_core().add_service("echo::flood::local_stream", service.get());
  service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::flood::local_stream");
}

static void run(kernel::module& module)
{ 
  network::local::stream::connection_service& service = static_cast<network::local::stream::connection_service&>(module.get_core().get_service("echo::flood::local_stream"));

  for (int count = module.get_option("count").get_int_value();
       count != 0;
       --count)
  {
    service.add_connect(module.get_option("local_socket").get_value());
  }
}

SCALESTACK_KERNEL_MODULE(options, start, stop, run, NULL);
