/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Stream Definitions
 */

#include "config.h"

#include <cstring>

#include <scalestack/echo/flood/flood.h>
#include <scalestack/echo/flood/stream.h>
#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>

namespace scalestack
{
namespace echo
{
namespace flood
{

/*
 * Public methods.
 */

stream::stream(network::stream_service& stream_service):
  network::stream(stream_service),
  _iterations(_module.get_option("iterations").get_size_value()),
  _read_offset(),
  _write_offset()
{
}

stream::~stream()
{
}

void stream::connected(void)
{
  flush_write();
}

size_t stream::read(uint8_t* buffer, size_t size)
{
  if (memcmp(random_buffer + _read_offset, buffer, size))
    _module.log_fatal(_("Read data did not match what was sent"));

  _read_offset += size;

  if (_read_offset == random_buffer_size)
  {
    if (_iterations == 0 || --_iterations > 0)
    {
      _read_offset = 0;
      _write_offset = 0;

      if (_module.get_option("reconnect").get_bool_value())
        reconnect();
      else
        flush_write();
    }
    else
      remove();
  }

  return size;
}

void stream::flush_write(void)
{
  _write_offset += write(random_buffer + _write_offset,
                         random_buffer_size - _write_offset,
                         true);
}

void stream::error(void)
{
  _module.log_error("Stream error");
  _module.get_core().shutdown();
  remove();
}

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */
