/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Stream Declarations
 */

#ifndef SCALESTACK_ECHO_FLOOD_STREAM_H
#define SCALESTACK_ECHO_FLOOD_STREAM_H

#include <scalestack/kernel/macros.h>
#include <scalestack/network/stream.h>

namespace scalestack
{
namespace kernel
{

class module;

} /* namespace kernel */

namespace echo
{
namespace flood
{

/**
 * Class for handling stream requests.
 */
class SCALESTACK_API stream: public network::stream
{
public:

  stream(network::stream_service& stream_service);

  ~stream();

  /**
   * See network::stream::connected().
   */
  void connected(void);

  /**
   * See network::stream::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * See network::stream::flush_write().
   */
  void flush_write(void);

  /**
   * See network::stream::error().
   */
  void error(void);

private:

  /**
   * Don't allow copying of objects.
   */
  stream(const stream&);

  /**
   * Don't allow assignment of objects.
   */
  stream& operator=(const stream&);

  size_t _iterations;
  size_t _read_offset;
  size_t _write_offset;
};

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_FLOOD_STREAM_H */
