/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood Stream Service Definitions
 */

#include "config.h"

#include <scalestack/echo/flood/stream.h>
#include <scalestack/echo/flood/stream_service.h>

namespace scalestack
{
namespace echo
{
namespace flood
{

/*
 * Public functions.
 */

stream_service::stream_service(kernel::module& module):
  network::stream_service(module),
  _count(0)
{
}

stream_service::~stream_service()
{
}

network::stream* stream_service::add_stream(void)
{
  _count++;
  return new stream(*this);
}

void stream_service::remove_stream(network::stream* stream)
{
  delete stream;

  _count--;
  if (_count == 0)
    _module.get_core().shutdown();
}

} /* namespace flood */
} /* namespace echo */
} /* namespace scalestack */
