/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Flood TCP Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/network/stream_service.h>
#include <scalestack/network/ip/tcp/connection_service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  /* Replace this option with default from network module */
  module.add_option("address", _("Host:port to connect to."), "HOST", ":7");
  //network::ip::tcp::connection_service::options(module, ":7");
  module.add_option("count",
                    _("Number of concurrent connections to open."),
                    "COUNT",
                    "1");
}

static void start(kernel::module& module)
{
  auto_ptr<network::ip::tcp::connection_service> service(new network::ip::tcp::connection_service(module, static_cast<network::stream_service&>(module.get_core().get_service("echo::flood::stream"))));

  module.get_core().add_service("echo::flood::tcp", service.get());

  service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::flood::tcp");
}

static void run(kernel::module& module)
{
  network::ip::tcp::connection_service& service = static_cast<network::ip::tcp::connection_service&>(module.get_core().get_service("echo::flood::tcp"));

  for (size_t count = module.get_option("count").get_size_value();
       count != 0;
       --count)
  {
    service.add_connection(module.get_option("address").get_value());
  }
}

SCALESTACK_KERNEL_MODULE(options, start, stop, run, NULL);
