/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Datagram Definitions
 */

#include "config.h"

#include <scalestack/echo/server/datagram.h>

namespace scalestack
{
namespace echo
{
namespace server
{

/*
 * Public methods.
 */

datagram::datagram(network::datagram_service& creator):
  network::datagram(creator),
  _size(),
  _buffer()
{
}

datagram::~datagram()
{
}

size_t datagram::receive(uint8_t* buffer, size_t size, struct sockaddr&, socklen_t)
{
  _size = size;
  _buffer = buffer;
  return _echo();
}

void datagram::flush_send(void)
{
  consume(_echo());
}

/*
 * Private methods.
 */

size_t datagram::_echo(void)
{
  return send(_buffer, _size);
}

} /* namespace server */
} /* namespace echo */
} /* namespace scalestack */
