/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Datagram Declarations
 */

#ifndef SCALESTACK_ECHO_SERVER_DATAGRAM_H
#define SCALESTACK_ECHO_SERVER_DATAGRAM_H

#include <scalestack/kernel/macros.h>
#include <scalestack/network/datagram.h>

namespace scalestack
{
namespace echo
{
namespace server
{

class SCALESTACK_API datagram: public network::datagram
{
public:

  datagram(network::datagram_service& creator);

  ~datagram();

  /**
   * See network::datagram::receive().
   */
  size_t receive(uint8_t* buffer,
                 size_t size,
                 struct sockaddr& peer,
                 socklen_t peer_size);

  /**
   * See network::datagram::flush_send().
   */
  void flush_send(void);

private:

  /**
   * Don't allow copying of objects.
   */
  datagram(const datagram&);

  /**
   * Don't allow assignment of objects.
   */
  datagram& operator=(const datagram&);

  /**
   * Echo the data received back to the sender. This is used by both the
   * receive() and flush_send() methods.
   */
  SCALESTACK_LOCAL
  size_t _echo(void);

  size_t _size;
  uint8_t* _buffer;
};

} /* namespace server */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_SERVER_DATAGRAM_H */
