/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Local Stream Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/network/stream_service.h>
#include <scalestack/network/local/stream/connection_service.h>
#include <scalestack/network/local/stream/listener_service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  network::local::stream::listener_options(module, "/tmp/echoserver.socket");
}

static void start(kernel::module& module)
{
  auto_ptr<network::local::stream::connection_service> connection_service(new network::local::stream::connection_service(module, static_cast<network::stream_service&>(module.get_core().get_service("echo::server::stream"))));

  module.get_core().add_service("echo::server::local_stream::connection",
                                connection_service.get());

  auto_ptr<network::local::stream::listener_service> listener_service(new network::local::stream::listener_service(module, *(connection_service.get())));

  module.get_core().add_service("echo::server::local_stream::listener",
                                listener_service.get());

  listener_service.release();
  connection_service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::server::local_stream::listener");
  delete module.get_core().remove_service("echo::server::local_stream::connection");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
