/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Unix Domain Socket Stream Test
 */

#include "config.h"

#include <unistd.h>

#include <scalestack/kernel/test.h>

static const char* test_socket = ".echoserver.local.stream.test";

TEST_SUITE_BEGIN

TEST_BEGIN(default)
  unlink(test_socket);
  core.add_module_option_value("event::service", "", TEST_EVENT_SERVICE);
  core.add_module_option_value("echo::server::local_stream", "local_sockets", test_socket);
  core.add_module_option_value("echo::flood::local_stream", "local_socket", test_socket);
  core.set_signal_handlers();
  core.run();
TEST_END

TEST_BEGIN(iterations)
  unlink(test_socket);
  core.add_module_option_value("event::service", "", TEST_EVENT_SERVICE);
  core.add_module_option_value("echo::server::local_stream", "local_sockets", test_socket);
  core.add_module_option_value("echo::flood::local_stream", "local_socket", test_socket);
  core.add_module_option_value("echo::flood", "iterations", "5");
  core.set_signal_handlers();
  core.run();
TEST_END

TEST_BEGIN(reconnect)
  unlink(test_socket);
  core.add_module_option_value("event::service", "", TEST_EVENT_SERVICE);
  core.add_module_option_value("echo::server::local_stream", "local_sockets", test_socket);
  core.add_module_option_value("echo::flood::local_stream", "local_socket", test_socket);
  core.add_module_option_value("echo::flood", "iterations", "5");
  core.add_module_option_value("echo::flood", "reconnect", "True");
  core.set_signal_handlers();
  core.run();
TEST_END

TEST_SUITE_END
