/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/echo/server/datagram_service.h>
#include <scalestack/echo/server/stream_service.h>
#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace std;
using namespace scalestack;

static void start(kernel::module& module)
{
  auto_ptr<echo::server::datagram_service> datagram_service(new echo::server::datagram_service(module));

  module.get_core().add_service("echo::server::datagram",
                                datagram_service.get());

  auto_ptr<echo::server::stream_service> stream_service(new echo::server::stream_service(module));

  module.get_core().add_service("echo::server::stream", stream_service.get());

  datagram_service.release();
  stream_service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::server::datagram");
  delete module.get_core().remove_service("echo::server::stream");
}

SCALESTACK_KERNEL_MODULE(NULL, start, stop, NULL, NULL);
