/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Stream Definitions
 */

#include "config.h"

#include <scalestack/echo/server/stream.h>

namespace scalestack
{
namespace echo
{
namespace server
{

/*
 * Public methods.
 */

stream::stream(network::stream_service& creator):
  network::stream(creator),
  _size(),
  _buffer()
{
}

stream::~stream()
{
}

size_t stream::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _echo();
}

void stream::flush_write(void)
{
  consume(_echo());
}

/*
 * Private methods.
 */

size_t stream::_echo(void)
{
  size_t written = write(_buffer, _size, true);
  _size -= written;
  _buffer += written;
  return written;
}

} /* namespace server */
} /* namespace echo */
} /* namespace scalestack */
