/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Stream Declarations
 */

#ifndef SCALESTACK_ECHO_SERVER_STREAM_H
#define SCALESTACK_ECHO_SERVER_STREAM_H

#include <scalestack/kernel/macros.h>
#include <scalestack/network/stream.h>

namespace scalestack
{
namespace echo
{
namespace server
{

class SCALESTACK_API stream: public network::stream
{
public:

  stream(network::stream_service& creator);

  ~stream();

  /**
   * See network::stream::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * See network::stream::flush_write().
   */
  void flush_write(void);

private:

  /**
   * Don't allow copying of objects.
   */
  stream(const stream&);

  /**
   * Don't allow assignment of objects.
   */
  stream& operator=(const stream&);

  /**
   * Echo the data read back to the client. This is used by both the read()
   * and flush_write() methods.
   */
  SCALESTACK_LOCAL
  size_t _echo(void);

  size_t _size;
  uint8_t* _buffer;
};

} /* namespace server */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_SERVER_STREAM_H */
