/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server Stream Service Declarations
 */

#ifndef SCALESTACK_ECHO_SERVER_STREAM_SERVICE_H
#define SCALESTACK_ECHO_SERVER_STREAM_SERVICE_H

#include <scalestack/network/stream_service.h>

namespace scalestack
{
namespace echo
{
namespace server
{

class stream_service: public network::stream_service
{
public:

  stream_service(kernel::module& module);

  ~stream_service();

  /**
   * See network::stream_service::add_stream().
   */
  network::stream *add_stream(void);

  /**
   * See network::stream_service::remove_stream().
   */
  void remove_stream(network::stream* stream);

private:

  /**
   * Don't allow copying of objects.
   */
  stream_service(const stream_service&);

  /**
   * Don't allow assignment of objects.
   */
  stream_service& operator=(const stream_service&);
};

} /* namespace server */
} /* namespace echo */
} /* namespace scalestack */

#endif /* SCALESTACK_ECHO_SERVER_STREAM_SERVICE_H */
