/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server TCP Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/network/stream_service.h>
#include <scalestack/network/ip/tcp/connection_service.h>
#include <scalestack/network/ip/tcp/listener_service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  network::ip::tcp::listener_service::options(module, ":7");
}

static void start(kernel::module& module)
{
  auto_ptr<network::ip::tcp::connection_service> connection_service(new network::ip::tcp::connection_service(module, static_cast<network::stream_service&>(module.get_core().get_service("echo::server::stream"))));

  module.get_core().add_service("echo::server::tcp::connection",
                                connection_service.get());

  auto_ptr<network::ip::tcp::listener_service> listener_service(new network::ip::tcp::listener_service(module, *(connection_service.get())));

  module.get_core().add_service("echo::server::tcp::listener",
                                listener_service.get());

  listener_service->add_listeners();
  listener_service.release();
  connection_service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::server::tcp::listener");
  delete module.get_core().remove_service("echo::server::tcp::connection");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
