/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Echo Server UDP Module Definitions
 */

#include "config.h"

#include <memory>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/network/datagram_service.h>
#include <scalestack/network/ip/udp/socket_service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  network::ip::udp::socket_service::options(module, ":7");
}

static void start(kernel::module& module)
{
  auto_ptr<network::ip::udp::socket_service> service(new network::ip::udp::socket_service(module, static_cast<network::datagram_service&>(module.get_core().get_service("echo::server::datagram"))));

  service->add_sockets();
  module.get_core().add_service("echo::server::udp", service.get());
  service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("echo::server::udp");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
