/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief libevent Event Handler Service Provider Declarations
 */

#ifndef SCALESTACK_EVENT_LIBEVENT_HANDLER_SERVICE_PROVIDER_H
#define SCALESTACK_EVENT_LIBEVENT_HANDLER_SERVICE_PROVIDER_H

#include <pthread.h>
#include <set>

#include <scalestack/event/handler_service.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace event
{
namespace libevent
{

class service;

/**
 * This is the handler service provider implementation.
 */
class SCALESTACK_API handler_service_provider: public event::handler_service_provider
{
public:

  handler_service_provider(kernel::module& module,
                           event::handler_service& handler_service);

  virtual ~handler_service_provider();

  /**
   * See event::handler_service::add_handler().
   */
  void add_handler(handler* handler);

  /**
   * See event::handler_service::remove_handler().
   */
  void remove_handler(handler* handler);

  /**
   * See event::handler_service::shutdown().
   */
  void shutdown(void);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  handler_service_provider(const handler_service_provider&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  handler_service_provider& operator=(const handler_service_provider&);

  typedef std::set<handler*> handlers;

  bool _shutdown;
  size_t _handler_count;
  libevent::service& _service;
  pthread_mutex_t _handler_mutex;
  int _notification_pipe[2];
  handlers _handlers;
};

} /* namespace libevent */
} /* namespace event */
} /* namespace scalestack */

#endif /* SCALESTACK_EVENT_LIBEVENT_HANDLER_SERVICE_PROVIDER_H */
