/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief libevent Event Module Definitions
 */

#include "config.h"

#include <exception>
#include <memory>

#include <scalestack/event/libevent/service.h>
#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  module.add_option("threads",
                    "Number of event threads to use.",
                    "THREADS",
                    "1");
  module.add_option("thread_stack_size",
                    "Size of the stack for threads (0 means default).",
                    "STACKSIZE",
                    "0");
}

static void start(kernel::module& module)
{
  auto_ptr<event::libevent::service> service(new event::libevent::service(module));
  module.get_core().add_service("event", service.get());
  service.release();
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("event");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
