/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Handler Declarations
 */

#ifndef SCALESTACK_EVENT_TEST_HANDLER_H
#define SCALESTACK_EVENT_TEST_HANDLER_H

#include <scalestack/event/handler.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace event
{
namespace test
{

class SCALESTACK_API handler: public event::handler
{
public:

  handler(handler_service& handler_service);

  ~handler();

  void start(void);

  void timer_expired(void);

  void read_ready(int file_descriptor);

  void write_ready(int file_descriptor);

  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  handler(const handler&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  handler& operator=(const handler&);
};

} /* namespace test */
} /* namespace event */
} /* namespace scalestack */

#endif /* SCALESTACK_EVENT_TEST_HANDLER_H */
