/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Handler Provider Declarations
 */

#ifndef SCALESTACK_EVENT_TEST_HANDLER_PROVIDER_H
#define SCALESTACK_EVENT_TEST_HANDLER_PROVIDER_H

#include <scalestack/event/handler.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace event
{
namespace test
{

class handler_service_provider;

class SCALESTACK_API handler_provider: public event::handler_provider
{
public:

  handler_provider(handler_service_provider& handler_service_provider);

  ~handler_provider();

  void remove(void);

  void set_timer(time_t timer);

  void set_file_descriptor(int file_descriptor);

  void watch_read(void);

  void watch_write(void);

  void run_now(void);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  handler_provider(const handler_provider&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  handler_provider& operator=(const handler_provider&);

  handler_service_provider& _handler_service_provider;
};

} /* namespace test */
} /* namespace event */
} /* namespace scalestack */

#endif /* SCALESTACK_EVENT_TEST_HANDLER_PROVIDER_H */
