/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Service Declarations
 */

#ifndef SCALESTACK_EVENT_TEST_HANDLER_SERVICE_H
#define SCALESTACK_EVENT_TEST_HANDLER_SERVICE_H

#include <scalestack/event/handler_service.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace event
{
namespace test
{

class SCALESTACK_API handler_service: public event::handler_service
{
public:

  handler_service(kernel::module& module);

  ~handler_service();

  /**
   * See event::handler_service::remove_handler().
   */
  void remove_handler(event::handler* handler);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  handler_service(const handler_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  handler_service& operator=(const handler_service&);
};

} /* namespace test */
} /* namespace event */
} /* namespace scalestack */

#endif /* SCALESTACK_EVENT_TEST_HANDLER_SERVICE_H */
