/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Service Provider Definitions
 */

#include "config.h"

#include <scalestack/event/test/handler_provider.h>
#include <scalestack/event/test/handler_service_provider.h>

namespace scalestack
{
namespace event
{
namespace test
{

/*
 * Public methods.
 */

handler_service_provider::handler_service_provider(kernel::module& module, event::handler_service& handler_service):
  event::handler_service_provider(module, handler_service),
  _handler_provider()
{
}

handler_service_provider::~handler_service_provider()
{
}

void handler_service_provider::add_handler(event::handler* handler)
{
  handler_provider* handler_provider = new test::handler_provider(*this);
  handler_provider->set_handler(handler);
  handler_provider->start();
}

void handler_service_provider::shutdown(void)
{
  remove_handler(_handler_provider->get_handler());
  delete _handler_provider;
}

void handler_service_provider::queue_to_remove(handler_provider* handler_provider)
{
  _handler_provider = handler_provider;
}

} /* namespace test */
} /* namespace event */
} /* namespace scalestack */
