/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Service Provider Declarations
 */

#ifndef SCALESTACK_EVENT_TEST_HANDLER_SERVICE_PROVIDER_H
#define SCALESTACK_EVENT_TEST_HANDLER_SERVICE_PROVIDER_H

#include <scalestack/event/handler_service.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace event
{
namespace test
{

class handler_provider;

class SCALESTACK_API handler_service_provider: public event::handler_service_provider
{
public:

  handler_service_provider(kernel::module& module,
                           event::handler_service& handler_service);

  ~handler_service_provider();

  /**
   * See event::handler_service::add_handler().
   */
  void add_handler(event::handler* handler);

  /**
   * See event::handler_service::shutdown().
   */
  void shutdown(void);

  /**
   * Queue a handler provider to be removed.
   */
  void queue_to_remove(handler_provider* handler_provider);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  handler_service_provider(const handler_service_provider&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  handler_service_provider& operator=(const handler_service_provider&);

  handler_provider* _handler_provider;
};

} /* namespace test */
} /* namespace event */
} /* namespace scalestack */

#endif /* SCALESTACK_EVENT_TEST_HANDLER_SERVICE_PROVIDER_H */
