/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Event Test Module Definitions
 */

#include "config.h"

#include <scalestack/event/test/handler.h>
#include <scalestack/event/test/handler_service.h>
#include <scalestack/event/test/service.h>
#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace std;
using namespace scalestack;

static void start(kernel::module& module)
{
  event::test::service* test_service = new event::test::service(module);
  module.get_core().add_service("event", test_service);

  event::test::handler_service* test_handler_service = new event::test::handler_service(module);
  module.get_core().add_service("event::test", test_handler_service);
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("event::test");
  delete module.get_core().remove_service("event");
}

static void run(kernel::module& module)
{
  event::test::handler_service& test_handler_service = static_cast<event::test::handler_service&>(module.get_core().get_service("event::test"));

  test_handler_service.add_handler(new event::test::handler(test_handler_service));

  module.get_core().shutdown();
}

SCALESTACK_KERNEL_MODULE(NULL, start, stop, run, NULL);
