/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Delete Pointer Function Object
 */

#ifndef SCALESTACK_KERNEL_DELETE_POINTER_H
#define SCALESTACK_KERNEL_DELETE_POINTER_H

#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace kernel
{

/**
 * Function object to use with containers of pointers when the objects should
 * be deleted.
 */
class SCALESTACK_LOCAL delete_pointer
{
public:

  /**
   * Delete the pointer passed in as an argument.
   */
  template<typename T>
  void operator()(const T* pointer) const
  {
    delete pointer;
  }
};

} /* namespace kernel */
} /* namespace scalestack */

#endif /* SCALESTACK_KERNEL_DELETE_POINTER_H */
