/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Kernel Exception Declarations
 */

#ifndef SCALESTACK_KERNEL_EXCEPTION_H
#define SCALESTACK_KERNEL_EXCEPTION_H

#include <exception>

#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace kernel
{

/**
 * This class allows exceptions to be thrown with custom error messages
 * without the possibility of throwing any exceptions itself.
 */
class SCALESTACK_API exception: public std::exception
{
public:

  /**
   * This constructor formats the context and message into a static buffer.
   *
   * @param[in] context Context where the exception occurred.
   * @param[in] message Message describing the exception.
   */
  exception(const char* context, const char* message) throw();

  /**
   * Get the exception message describing what happened.
   *
   * @return Static buffer containing the message.
   */
  const char* what() const throw();

private:

  /* Use a static sized buffer so there is no risk of throwing exceptions. */
  char _message[SCALESTACK_MAX_MESSAGE_SIZE];
};

} /* namespace kernel */
} /* namespace scalestack */

#endif /* SCALESTACK_KERNEL_EXCEPTION_H */
