/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Log Console Definitions
 */

#include "config.h"

#include <cerrno>
#include <cstdio>
#include <stdint.h>
#include <unistd.h>

#include <scalestack/kernel/log_console.h>

using namespace std;

namespace scalestack
{
namespace kernel
{

/*
 * Public methods.
 */

log_console::log_console()
{
}

void log_console::write_log(log::level log_level,
                            const char* context,
                            const char* message)
{
  /* Another buffer is used here so we can have one atomic write to stdout. */
  char buffer[SCALESTACK_MAX_MESSAGE_SIZE];
  uint32_t size = snprintf(buffer, SCALESTACK_MAX_MESSAGE_SIZE, "%6s [%s] %s\n",
                           get_level_string(log_level), context, message);
  if (size > SCALESTACK_MAX_MESSAGE_SIZE)
    size = SCALESTACK_MAX_MESSAGE_SIZE;
  if (write(1, buffer, size) == -1 && errno != EPIPE)
    fprintf(stderr, _("Could not write to console: %d\n"), errno);
}

} /* namespace kernel */
} /* namespace scalestack */
