/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Log Console Declarations
 */

#ifndef SCALESTACK_KERNEL_LOG_CONSOLE_H
#define SCALESTACK_KERNEL_LOG_CONSOLE_H

#include <scalestack/kernel/log.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace kernel
{

/**
 * This is a kernel log implementation that writes to standard output.
 */
class SCALESTACK_API log_console: public log
{
public:

  log_console(void);

  /**
   * Write a log entry to standard output.
   *
   * @param[in] log_level Log level for this message.
   * @param[in] context Context string for this message.
   * @param[in] message Message to log.
   */
  void write_log(log::level log_level,
                 const char* context,
                 const char* message);

private:

  /**
   * Don't allow copying of objects.
   */
  log_console(const log_console&);

  /**
   * Don't allow assignment of objects.
   */
  log_console& operator=(const log_console&);
};

} /* namespace kernel */
} /* namespace scalestack */

#endif /* SCALESTACK_KERNEL_LOG_CONSOLE_H */
