/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Service Declarations
 */

#ifndef SCALESTACK_KERNEL_SERVICE_H
#define SCALESTACK_KERNEL_SERVICE_H

#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace kernel
{

class module;

/**
 * This is the base class that all services should inherit from. It is a
 * common type to provide some level of safety when adding services to the
 * kernel core.
 */
class SCALESTACK_API service
{
public:

  service(kernel::module& module);

  virtual ~service();

  /**
   * Get the module associated with this service.
   */
  kernel::module& get_module(void);

protected:

  kernel::module& _module;

private:

  /**
   * Don't allow copying of objects.
   */
  service(const service&);

  /**
   * Don't allow assignment of objects.
   */
  service& operator=(const service&);
};

/*
 * Public methods.
 */

inline service::service(kernel::module& module):
  _module(module)
{
}

inline service::~service()
{
}

inline kernel::module& service::get_module(void)
{
  return _module;
}

} /* namespace kernel */
} /* namespace scalestack */

#endif /* SCALESTACK_KERNEL_SERVICE_H */
