/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Module Child Module Definitions
 */

#include "config.h"

#include <cassert>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace scalestack;

static void start(kernel::module& module)
{
  assert(module.get_name() == "kernel::test::module_child");
  assert(module.get_pathname() != "");
  assert(module.get_version() == "module_child version");
  assert(module.get_author() == "module_child author");
  assert(module.get_title() == "module_child title");
  assert(module.get_license() == "module_child_license");
  assert(module.get_dependencies() == "kernel::test::module_parent");
  assert(module.get_parent_count() == 1);
  assert(module.is_loaded() == true);

  kernel::module& parent = module.get_core().get_module("kernel::test::module_parent");
  assert(parent.get_title() == "module_parent title");
  assert(parent.is_loaded() == true);

  kernel::module& grand_parent = module.get_core().get_module("kernel::test::module_grand_parent");
  assert(grand_parent.get_title() == "module_grand_parent title");
  assert(grand_parent.is_loaded() == true);
}

SCALESTACK_KERNEL_MODULE(NULL, start, NULL, NULL, NULL);
