/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Module Grand Parent Module Definitions
 */

#include "config.h"

#include <cassert>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>

using namespace scalestack;

static void options(kernel::module& module)
{
  assert(module.is_loaded() == true);
}

static void start(kernel::module& module)
{
  assert(module.get_name() == "kernel::test::module_grand_parent");
  assert(module.get_pathname() != "");
  assert(module.get_version() == "module_grand_parent version");
  assert(module.get_author() == "module_grand_parent author");
  assert(module.get_title() == "module_grand_parent title");
  assert(module.get_license() == "module_grand_parent_license");
  assert(module.get_dependencies() == "");
  assert(module.get_parent_count() == 0);
  assert(module.is_loaded() == true);
}

static void stop(kernel::module& module)
{
  assert(module.is_loaded() == true);
}

static void run(kernel::module& module)
{
  assert(module.is_loaded() == true);
  module.get_core().shutdown();
}

static void reconfigure(kernel::module& module)
{
  assert(module.is_loaded() == true);
}

SCALESTACK_KERNEL_MODULE(options, start, stop, run, reconfigure);
