/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief option Tests
 */

#include "config.h"

#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/kernel/test.h>

TEST_SUITE_BEGIN

TEST_BEGIN(constructor_true)
  kernel::module& module = core.get_or_add_module("module");
  kernel::option& option = module.add_option("option", "Help", "BOOL", "true");
  assert(&option.get_module() == &module);
  assert(option.get_name() == "option");
  assert(option.get_help() == "Help");
  assert(option.get_value_name() == "BOOL");
  assert(option.get_default_value() == "true");
  assert(option.get_value() == "true");
  assert(option.get_bool_value() == true);
  assert(option.is_valid() == true);
TEST_END

TEST_BEGIN(constructor_false)
  kernel::module& module = core.get_or_add_module("module");
  kernel::option& option = module.add_option("option", "Help", "BOOL", "false");
  assert(&option.get_module() == &module);
  assert(option.get_name() == "option");
  assert(option.get_help() == "Help");
  assert(option.get_value_name() == "BOOL");
  assert(option.get_default_value() == "false");
  assert(option.get_value() == "false");
  assert(option.get_bool_value() == false);
  assert(option.is_valid() == true);
TEST_END

TEST_BEGIN(constructor_value)
  kernel::module& module = core.get_or_add_module("module");
  kernel::option& option = module.add_option("option",
                                             "Help",
                                             "VALUE_NAME",
                                             "default_value");
  assert(&option.get_module() == &module);
  assert(option.get_name() == "option");
  assert(option.get_help() == "Help");
  assert(option.get_value_name() == "VALUE_NAME");
  assert(option.get_default_value() == "default_value");
  assert(option.get_value() == "default_value");
  assert(option.is_valid() == true);
TEST_END

TEST_BEGIN(load_option)
  kernel::module& module = core.get_or_add_module("kernel::test::option_basic");
  core.add_module_option_value("kernel::test::option_basic", "option4", "");
  core.add_module_option_value("kernel::test::option_basic", "option5", "");
  core.add_module_option_value("kernel::test::option_basic", "option6", "value6");
  assert(module.get_option("option4").is_valid() == false);
  assert(module.get_option("option5").is_valid() == false);
  assert(module.get_option("option6").is_valid() == false);
  core.run();
  assert(module.get_option("option1").is_valid() == true);
  assert(module.get_option("option2").is_valid() == true);
  assert(module.get_option("option3").is_valid() == true);
  assert(module.get_option("option4").is_valid() == true);
  assert(module.get_option("option5").is_valid() == true);
  assert(module.get_option("option6").is_valid() == true);
TEST_END

TEST_BEGIN(bad_option)
  core.get_or_add_module("kernel::test::option_bad");
  TEST_EXCEPTION(core.run());
TEST_END

TEST_SUITE_END
