/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief option_basic module Definitions
 */

#include "config.h"

#include <cassert>

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>

using namespace scalestack;

static void options(kernel::module& module)
{
  module.add_option("option1", "Help1", "BOOL1", "true");
  module.add_option("option2", "Help2", "BOOL2", "false");
  module.add_option("option3", "Help3", "VALUE_NAME3", "default_value3");
  module.add_option("option4", "Help4", "BOOL4", "true");
  module.add_option("option5", "Help5", "BOOL5", "false");
  module.add_option("option6", "Help6", "VALUE_NAME6", "default_value6");
  module.add_option("option7", "Help7", "SIZE7", "7");
}

static void start(kernel::module& module)
{
  assert(module.get_option("option1").get_bool_value() == true);
  assert(module.get_option("option2").get_bool_value() == false);
  assert(module.get_option("option3").get_value() == "default_value3");
  assert(module.get_option("option4").get_bool_value() == true);
  assert(module.get_option("option5").get_bool_value() == true);
  assert(module.get_option("option6").get_value() == "value6");
  assert(module.get_option("option7").get_size_value() == 7);
}

static void run(kernel::module& module)
{
  module.get_core().shutdown();
}

SCALESTACK_KERNEL_MODULE(options, start, NULL, run, NULL);
