/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Datagram Service Declarations
 */

#ifndef SCALESTACK_NETWORK_DATAGRAM_SERVICE_H
#define SCALESTACK_NETWORK_DATAGRAM_SERVICE_H

#include <scalestack/kernel/macros.h>
#include <scalestack/kernel/service.h>

namespace scalestack
{
namespace network
{

class datagram;

class SCALESTACK_API datagram_service: public kernel::service
{
public:

  datagram_service(kernel::module& module);

  virtual ~datagram_service();

  /**
   * Get a new datagram from a derived class.
   *
   * @return Datagram that was added.
   */
  virtual network::datagram* add_datagram(void) = 0;

  /**
   * Remove a datagram that was added in add_datagram().
   *
   * @param[in] datagram The datagram to remove.
   */
  virtual void remove_datagram(network::datagram* datagram) = 0;

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  datagram_service(const datagram_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  datagram_service& operator=(const datagram_service&);
};

/*
 * Public methods.
 */

inline datagram_service::datagram_service(kernel::module& module):
  kernel::service(module)
{
}

inline datagram_service::~datagram_service()
{
}

} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_DATAGRAM_SERVICE_H */
