/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network IP TCP Connection Declarations
 */

#ifndef SCALESTACK_NETWORK_IP_TCP_CONNECTION_H
#define SCALESTACK_NETWORK_IP_TCP_CONNECTION_H

#include <string>

#include <scalestack/kernel/macros.h>
#include <scalestack/network/stream.h>

namespace scalestack
{
namespace network
{
namespace ip
{
namespace tcp
{

class connection_service;

class SCALESTACK_API connection: public network::stream_provider
{
public:

  connection(connection_service& connection_service);

  ~connection();

  /**
   * Set the file descriptor to use for this connection. This is usually
   * the result of an accepted connection.
   *
   * @param[in] file_descriptor File descriptor to set.
   */
  void set_file_descriptor(int file_descriptor);

  /**
   * Connect to a remote host.
   *
   * @param[in] address Host:port to connect to.
   */
  void connect(const std::string& address);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  connection(const connection&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  connection& operator=(const connection&);
};

} /* namespace tcp */
} /* namespace ip */
} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_IP_TCP_CONNECTION_H */
