/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network IP TCP Listener Definitions
 */

#include "config.h"

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>

#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/network/ip/tcp/listener.h>
#include <scalestack/network/ip/tcp/listener_service.h>

using namespace std;

namespace scalestack
{
namespace network
{
namespace ip
{
namespace tcp
{

/*
 * Public methods.
 */

listener::listener(listener_service& listener_service,
                   const struct addrinfo& address):
  network::listener(listener_service)
{
  char host_name[NI_MAXHOST];
  char port_name[NI_MAXSERV];

  int return_code = getnameinfo(address.ai_addr,
                                address.ai_addrlen,
                                host_name, NI_MAXHOST,
                                port_name, NI_MAXSERV,
                                NI_NUMERICHOST | NI_NUMERICSERV);
  if (return_code != 0)
  {
    _module.log_fatal(_("Error adding stream listener, getnameinfo: %s"),
                      gai_strerror(return_code));
  }

  _module.log_info(_("Trying to listen on %s:%s"), host_name, port_name);

  listen(address, _module.get_option("backlog").get_int_value());

  _module.log_notice(_("Listening on %s:%s"), host_name, port_name);

  _module.log_info(_("TCP listener constructed"));
}

listener::~listener()
{
  _module.log_info(_("TCP listener destructed"));
}

} /* namespace tcp */
} /* namespace ip */
} /* namespace network */
} /* namespace scalestack */
