/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network IP TCP Listener Declarations
 */

#ifndef SCALESTACK_NETWORK_IP_TCP_LISTENER_H
#define SCALESTACK_NETWORK_IP_TCP_LISTENER_H

#include <scalestack/network/listener.h>

struct addrinfo;

namespace scalestack
{
namespace network
{
namespace ip
{
namespace tcp
{

class listener_service;

class listener: public network::listener
{
public:

  /**
   * Constructor that starts listening on a new address.
   *
   * @param[in] listener_service Service this listener is associated with.
   * @param[in] address Address information to use for socket.
   */
  listener(listener_service& listener_service, const struct addrinfo& address);

  ~listener();

private:

  /**
   * Don't allow copying of objects.
   */
  listener(const listener&);

  /**
   * Don't allow assignment of objects.
   */
  listener& operator=(const listener&);
};

} /* namespace tcp */
} /* namespace ip */
} /* namespace socket_stream */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_IP_TCP_LISTENER_H */
