/*
 * Scale Stack
 *
 * Copyright 2010 Chris Behrens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network IP UDP Socket Declarations
 */

#ifndef SCALESTACK_NETWORK_IP_UDP_SOCKET_H
#define SCALESTACK_NETWORK_IP_UDP_SOCKET_H

#include <scalestack/kernel/macros.h>
#include <scalestack/network/datagram.h>

namespace scalestack
{
namespace network
{
namespace ip
{
namespace udp
{

class socket_service;

class SCALESTACK_API socket: public network::datagram_provider
{
public:

  socket(socket_service& socket_service);

  ~socket();

  /**
   * Set the file descriptor to use for this socket.
   *
   * @param[in] file_descriptor File descriptor to set.
   */
  void set_file_descriptor(int file_descriptor);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  socket(const socket&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  socket& operator=(const socket&);
};

} /* namespace udp */
} /* namespace ip */
} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_IP_UDP_SOCKET_H */
