/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Stream Listener Declarations
 */

#ifndef SCALESTACK_NETWORK_LISTENER_H
#define SCALESTACK_NETWORK_LISTENER_H

#include <scalestack/event/handler.h>
#include <scalestack/kernel/macros.h>

struct addrinfo;

namespace scalestack
{
namespace network
{

class listener_service;

class SCALESTACK_API listener: public event::handler
{
public:

  /**
   * Constructor, setup associated services.
   *
   * @param[in] listener_service Service this listener is associated with.
   */
  listener(listener_service& listener_service);

  virtual ~listener();

  /**
   * Bind and start listening on a socket.
   *
   * @param[in] local The local address to bind and listen on.
   * @param[in] backlog Maximum length of backlog queue, passed to listen
   *  system call.
   */
  void listen(const struct addrinfo& local, int backlog);

  /**
   * This is called when a stream is ready to be accepted on the listening
   * socket. See event::handler::read_ready()
   */
  virtual void read_ready(int file_descriptor);

private:

  /**
   * Don't allow copying of objects.
   */
  listener(const listener&);

  /**
   * Don't allow assignment of objects.
   */
  listener& operator=(const listener&);

  int _file_descriptor;
  listener_service& _listener_service;
};

} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LISTENER_H */
