/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Listener Service Declarations
 */

#ifndef SCALESTACK_NETWORK_LISTENER_SERVICE_H
#define SCALESTACK_NETWORK_LISTENER_SERVICE_H

#include <sys/socket.h>
#include <sys/types.h>

#include <scalestack/event/handler_service.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace network
{

class stream;

class SCALESTACK_API listener_service: public event::handler_service
{
public:

  listener_service(kernel::module& module);

  virtual ~listener_service();

  /**
   * Add a new stream that has been accepted.
   *
   * @param[in] file_descriptor File descriptor for the new stream.
   * @param[in] peer Address information for the remote peer.
   * @param[in] peer_size Size of peer structure.
   * @return Stream that was added.
   */
  virtual network::stream* add_accepted(int file_descriptor,
                                        struct sockaddr& peer,
                                        socklen_t peer_size) = 0;

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  listener_service(const listener_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  listener_service& operator=(const listener_service&);
};

/*
 * Public methods.
 */

inline listener_service::listener_service(kernel::module& module):
  event::handler_service(module)
{
}

inline listener_service::~listener_service()
{
}

} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LISTENER_SERVICE_H */
