/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Connection Definitions
 */

#include "config.h"

#include <cstring>

#include <scalestack/kernel/module.h>
#include <scalestack/network/local/stream/connection.h>
#include <scalestack/network/local/stream/connection_service.h>

using namespace std;

namespace scalestack
{
namespace network
{
namespace local
{
namespace stream
{

/*
 * Public methods.
 */

connection::connection(connection_service& connection_service):
  network::stream_provider(connection_service),
  _connect_sockaddr(),
  _connect_addrinfo()
{
  _connect_addrinfo.ai_family = AF_UNIX;
  _connect_addrinfo.ai_addr = reinterpret_cast<struct sockaddr*>(&_connect_sockaddr);
  _connect_addrinfo.ai_addrlen = sizeof(_connect_sockaddr);
}

connection::~connection()
{
}

void connection::set_file_descriptor(int file_descriptor)
{
  network::stream_provider::set_file_descriptor(file_descriptor);
}

void connection::connect(const std::string& local_socket)
{
  memset(&_connect_sockaddr, 0, sizeof(_connect_sockaddr));

  _connect_sockaddr.sun_family = AF_UNIX;
  strncpy(_connect_sockaddr.sun_path, local_socket.c_str(), sizeof(_connect_sockaddr.sun_path) - 1);
  _address_list = &_connect_addrinfo;

  try
  {
    network::stream_provider::connect();
  }
  catch(exception&)
  {
    error();
    return;
  }
}

} /* namespace stream */
} /* namespace local */
} /* namespace network */
} /* namespace scalestack */
