/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Connection Declarations
 */

#ifndef SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_H
#define SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_H

#include <netdb.h>
#include <string>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include <scalestack/event/handler.h>
#include <scalestack/kernel/macros.h>
#include <scalestack/network/stream.h>

namespace scalestack
{
namespace kernel
{

class module;

} /* namespace kernel */

namespace network
{
namespace local
{
namespace stream
{

class connection_service;

class SCALESTACK_API connection: public network::stream_provider
{
public:

  connection(connection_service& connection_service);

  ~connection();

  /**
   * Set the file descriptor to use for this connection. This is usually
   * the result of an accepted connection.
   *
   * @param[in] file_descriptor File descriptor to set.
   */
  void set_file_descriptor(int file_descriptor);

  /**
   * Connect to a remote host.
   *
   * @param[in] socket_name Socket to connect to.
   */
  void connect(const std::string& socket_name);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  connection(const connection&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  connection& operator=(const connection&);

  struct sockaddr_un _connect_sockaddr;
  struct addrinfo _connect_addrinfo;
};

} /* namespace stream */
} /* namespace local */
} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_H */
