/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Connection Service Declarations
 */

#ifndef SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_SERVICE_H
#define SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_SERVICE_H

#include <string>
#include <sys/socket.h>
#include <sys/types.h>

#include <scalestack/event/handler_service.h>
#include <scalestack/kernel/macros.h>

namespace scalestack
{
namespace network
{

class stream;
class stream_service;

namespace local
{
namespace stream
{

class connection;

class SCALESTACK_API connection_service: public event::handler_service
{
public:

  connection_service(kernel::module& module, stream_service& stream_service);

  virtual ~connection_service();

  /**
   * See stream_service::add_accepted().
   */
  network::stream* add_accepted(int file_descriptor,
                                struct sockaddr& peer,
                                socklen_t peer_size);

  /**
   * Add a new connection and connect to the given Local Domain Socket.
   *
   * @param[in] local_socket Local Domain Socket to connect to.
   * @return Connection that was added.
   */
  network::stream* add_connect(const std::string& local_socket);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  connection_service(const connection_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  connection_service& operator=(const connection_service&);

  /**
   * Add a new connection to use for accept() or connect().
   */
  SCALESTACK_LOCAL
  network::stream* _add_connection(connection** connection_ret);

  /**
   * See event::service::remove_handler().
   */
  void remove_handler(event::handler* handler);

  stream_service& _stream_service;
};

} /* namespace stream */
} /* namespace local */
} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LOCAL_STREAM_CONNECTION_SERVICE_H */
