/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Listener Definitions
 */

#include "config.h"

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/network/local/stream/listener.h>
#include <scalestack/network/local/stream/listener_service.h>

using namespace std;

namespace scalestack
{
namespace network
{
namespace local
{
namespace stream
{

/*
 * Public methods.
 */

listener::listener(listener_service& listener_service,
                   const std::string local_socket):
  network::listener(listener_service),
  _local_socket(local_socket)
{
  struct addrinfo addrinfo;
  struct sockaddr_un sun;

  memset(&sun, 0, sizeof(sun));
  sun.sun_family = AF_UNIX;
  /* leave the NULL at end of sun_path always */
  strncpy(sun.sun_path, _local_socket.c_str(), sizeof(sun.sun_path) - 1);

  memset(&addrinfo, 0, sizeof(addrinfo));
  addrinfo.ai_family = AF_UNIX;
  addrinfo.ai_socktype = SOCK_STREAM;
  addrinfo.ai_addr = reinterpret_cast<struct sockaddr*>(&sun);
  addrinfo.ai_addrlen = sizeof(sun);

  _module.log_info("Trying to listen on %s", _local_socket.c_str());

  listen(addrinfo, _module.get_option("local_backlog").get_int_value());

  _module.log_notice("Listening on %s", _local_socket.c_str());

  _module.log_info("network::local::stream::listener constructed");
}

listener::~listener()
{
  unlink(_local_socket.c_str());
  _module.log_info("network::local::stream::listener destructed");
}

} /* namespace stream */
} /* namespace local */
} /* namespace network */
} /* namespace scalestack */
