/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Listener Declarations
 */

#ifndef SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_H
#define SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_H

#include <string>

#include <scalestack/network/listener.h>

namespace scalestack
{
namespace network
{
namespace local
{
namespace stream
{

class listener_service;

class listener: public network::listener
{
public:

  /**
   * Constructor that starts listening on the address described in addrinfo.
   *
   * @param[in] listener_service Service this listener is associated with.
   * @param[in] local_socket Address information to use for socket.
   */
  listener(listener_service& listener_service, std::string local_socket);

  ~listener();

private:

  /**
   * Don't allow copying of objects.
   */
  listener(const listener&);

  /**
   * Don't allow assignment of objects.
   */
  listener& operator=(const listener&);

  std::string _local_socket;
};

} /* namespace stream */
} /* namespace local */
} /* namespace socket_stream */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_H */
