/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Local Domain Socket Stream Listener Service Declarations
 */

#ifndef SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_SERVICE_H
#define SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_SERVICE_H

#include <string>

#include <scalestack/network/listener_service.h>
#include <scalestack/network/local/common.h>

namespace scalestack
{
namespace network
{
namespace local
{
namespace stream
{

class listener;
class connection_service;

/**
 * Add common options for Connection listen sockets.
 *
 * @param[in] module Module that the service is associated with.
 * @param[in] port Default port to use.
 */
SCALESTACK_API
void listener_options(kernel::module& module, const char* port);

class SCALESTACK_API listener_service: public network::listener_service,
                                       public local::common
{
public:

  listener_service(kernel::module& module,
                   connection_service& connection_service);

  ~listener_service();

  /**
   * See network::listener_service::add_accepted().
   */
  network::stream* add_accepted(int file_descriptor,
                                struct sockaddr& peer,
                                socklen_t peer_size);

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  listener_service(const listener_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  listener_service& operator=(const listener_service&);

  /**
   * Add listeners for the host/port pair.
   */
  SCALESTACK_LOCAL
  void _add_listener(std::string local_socket);

  /**
   * See event::service::remove_handler().
   */
  SCALESTACK_LOCAL
  void remove_handler(event::handler* handler);

  connection_service& _connection_service;
};

} /* namespace stream */
} /* namespace local */
} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_LOCAL_STREAM_LISTENER_SERVICE_H */
