/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Network Stream Service Declarations
 */

#ifndef SCALESTACK_NETWORK_STREAM_SERVICE_H
#define SCALESTACK_NETWORK_STREAM_SERVICE_H

#include <scalestack/kernel/macros.h>
#include <scalestack/kernel/service.h>

namespace scalestack
{
namespace network
{

class stream;

class SCALESTACK_API stream_service: public kernel::service
{
public:

  stream_service(kernel::module& module);

  virtual ~stream_service();

  /**
   * Get a new stream from a derived class.
   *
   * @return Stream that was added.
   */
  virtual network::stream* add_stream(void) = 0;

  /**
   * Remove a stream that was added in add_stream().
   *
   * @param[in] stream The stream to remove.
   */
  virtual void remove_stream(network::stream* stream) = 0;

private:

  /**
   * Don't allow copying of objects.
   */
  SCALESTACK_LOCAL
  stream_service(const stream_service&);

  /**
   * Don't allow assignment of objects.
   */
  SCALESTACK_LOCAL
  stream_service& operator=(const stream_service&);
};

/*
 * Public methods.
 */

inline stream_service::stream_service(kernel::module& module):
  kernel::service(module)
{
}

inline stream_service::~stream_service()
{
}

} /* namespace network */
} /* namespace scalestack */

#endif /* SCALESTACK_NETWORK_STREAM_SERVICE_H */
