/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Client Definitions
 */

#include "config.h"

#include <scalestack/kernel/module.h>
#include <scalestack/proxy/client.h>
#include <scalestack/proxy/client_service.h>
#include <scalestack/proxy/connection.h>

namespace scalestack
{
namespace proxy
{

/*
 * Public methods.
 */

client::client(client_service &client_service):
  network::stream(client_service),
  _shutdown(false),
  _size(),
  _buffer(),
  _connection()
{
}

void client::set_connection(proxy::connection* connection)
{
  if (connection == NULL)
  {
    _shutdown = true;
    run_now();
    return;
  }

  _connection = connection;
}

void client::connected(void)
{
  _connection->continue_write();
}

size_t client::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _connection_write();
}

void client::continue_write(void)
{
  consume(_connection_write());
}

void client::flush_write(void)
{
  _connection->continue_write();
}

void client::error(void)
{
  _connection->error();
}

void client::run(void)
{
  if (_shutdown)
  {
    _connection->run_now();
    remove();
  }
}

/*
 * Private methods.
 */

size_t client::_connection_write(void)
{
  size_t written = _connection->write(_buffer, _size, true);
  _size -= written;
  _buffer += written;
  return written;
}

} /* namespace proxy */
} /* namespace scalestack */
