/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Client Declarations
 */

#ifndef SCALESTACK_PROXY_CLIENT_H
#define SCALESTACK_PROXY_CLIENT_H

#include <scalestack/network/stream.h>

namespace scalestack
{
namespace proxy
{

class client_service;
class connection;

class client: public network::stream
{
public:

  client(client_service& client_service);

  /**
   * Set the connection that uses this client.
   */
  void set_connection(proxy::connection* connection);

  /**
   * Once connected, starting writing data from attached connection.
   */
  void connected(void);

  /**
   * See Socket_tCP::connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the connection from the read buffer.
   */
  void continue_write(void);

  /**
   * See Socket_tCP::connection::flush_write().
   */
  void flush_write(void);

  /**
   * On error, notify attached connection to shutdown.
   */
  void error(void);

  /**
   * Process requests from attached connection on this clients thread.
   */
  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  client(const client&);

  /**
   * Don't allow assignment of objects.
   */
  client& operator=(const client&);

  /**
   * Write data to the connection. This is used by both the read() and
   * continue_write() methods.
   */
  size_t _connection_write(void);

  bool _shutdown;
  size_t _size;
  uint8_t* _buffer;
  proxy::connection* _connection;
};

} /* namespace proxy */
} /* namespace scalestack */

#endif /* SCALESTACK_PROXY_CLIENT_H */
