/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Client Service Definitions
 */

#include "config.h"

#include <scalestack/kernel/module.h>
#include <scalestack/kernel/option.h>
#include <scalestack/proxy/client.h>
#include <scalestack/proxy/client_service.h>

using namespace std;

namespace scalestack
{
namespace proxy
{

/*
 * Public functions.
 */

client_service::client_service(kernel::module& module):
  network::stream_service(module),
  _connection_service(module, *this)
{
}

client_service::~client_service()
{
}

network::stream* client_service::add_stream(void)
{
  return new client(*this);
}

void client_service::remove_stream(network::stream* stream)
{
  delete stream;
}

client* client_service::get_client(connection* connection)
{
  client* client = static_cast<proxy::client*>(_connection_service.add_connection(_module.get_option("client_address").get_value()));
  client->set_connection(connection);
  return client;
}

} /* namespace proxy */
} /* namespace scalestack */
