/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Connection Definitions
 */

#include "config.h"

#include <scalestack/kernel/module.h>
#include <scalestack/proxy/client.h>
#include <scalestack/proxy/client_service.h>
#include <scalestack/proxy/connection.h>
#include <scalestack/proxy/service.h>

namespace scalestack
{
namespace proxy
{

/*
 * Public methods.
 */

connection::connection(service& service, client_service& client_service):
  network::stream(service),
  _shutdown(false),
  _shutdown_write(false),
  _size(),
  _buffer(),
  _client(client_service.get_client(this))
{
}

size_t connection::read(uint8_t* buffer, size_t size)
{
  _size = size;
  _buffer = buffer;
  return _client_write();
}

void connection::continue_write(void)
{
  consume(_client_write());
}

void connection::flush_write(void)
{
  if (!_shutdown)
    _client->continue_write();
}

void connection::read_eOF(void)
{
  if (_shutdown)
    error();
  else
  {
    if (_size == 0)
      _client->shutdown_write();
    else
      _shutdown_write = true;
  }
}

void connection::error(void)
{
  _shutdown = true;
  run_now();
}

void connection::run(void)
{
  if (_client == NULL)
  {
    remove();
    return;
  }

  if (_shutdown)
  {
    _client->set_connection(NULL);
    _client = NULL;
    return;
  }
}

/*
 * Private methods.
 */

size_t connection::_client_write(void)
{
  if (_shutdown)
    return 0;

  size_t written = _client->write(_buffer, _size, true);
  _size -= written;
  _buffer += written;

  if (_shutdown_write && _size == 0)
    _client->shutdown_write();

  return written;
}

} /* namespace proxy */
} /* namespace scalestack */
