/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Connection Declarations
 */

#ifndef SCALESTACK_PROXY_CONNECTION_H
#define SCALESTACK_PROXY_CONNECTION_H

#include <scalestack/network/stream.h>

namespace scalestack
{
namespace proxy
{

class client;
class client_service;
class service;

class connection: public network::stream
{
public:

  connection(service& service, client_service& client_service);

  /**
   * See Socket_tCP::connection::read().
   */
  size_t read(uint8_t* buffer, size_t size);

  /**
   * Continue writing to the client from the read buffer.
   */
  void continue_write(void);

  /**
   * See Socket_tCP::connection::flush_write().
   */
  void flush_write(void);

  /**
   * Notify the client to shutdown the write socket if needed.
   */
  void read_eOF(void);

  /**
   * Start shutdown process.
   */
  void error(void);

  /**
   * Process shutdown requests, detaching from client.
   */
  void run(void);

private:

  /**
   * Don't allow copying of objects.
   */
  connection(const connection&);

  /**
   * Don't allow assignment of objects.
   */
  connection& operator=(const connection&);

  /**
   * Write data to the client. This is used by both the read() and
   * continue_write() methods.
   */
  size_t _client_write(void);

  bool _shutdown;
  bool _shutdown_write;
  size_t _size;
  uint8_t* _buffer;
  client* _client;
};

} /* namespace proxy */
} /* namespace scalestack */

#endif /* SCALESTACK_PROXY_CONNECTION_H */
