/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Module Definitions
 */

#include "config.h"

#include <scalestack/kernel/core.h>
#include <scalestack/kernel/module.h>
#include <scalestack/proxy/service.h>

using namespace std;
using namespace scalestack;

static void options(kernel::module& module)
{
  network::ip::tcp::listener_service::options(module, ":7");
  module.add_option("client_address", "Host:port to proxy to.", "HOST", "127.0.0.1:12345");
}

static void start(kernel::module& module)
{
  proxy::service* service = new proxy::service(module);
  try
  {
    module.get_core().add_service("proxy", service);
  }
  catch (exception&)
  {
    delete service;
    throw;
  }
}

static void stop(kernel::module& module)
{
  delete module.get_core().remove_service("proxy");
}

SCALESTACK_KERNEL_MODULE(options, start, stop, NULL, NULL);
