/*
 * Scale Stack
 *
 * Copyright 2010 Eric Day
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file
 * @brief Proxy Service Declarations
 */

#ifndef SCALESTACK_PROXY_SERVICE_H
#define SCALESTACK_PROXY_SERVICE_H

#include <scalestack/proxy/client_service.h>
#include <scalestack/network/ip/tcp/connection_service.h>
#include <scalestack/network/ip/tcp/listener_service.h>
#include <scalestack/network/stream_service.h>

namespace scalestack
{
namespace proxy
{

class client;
class connection;

class service: public network::stream_service
{
public:

  service(kernel::module& module);

  ~service();

  /**
   * See network::stream_service::add_stream().
   */
  network::stream* add_stream(void);

  /**
   * See network::stream_service::remove_stream().
   */
  void remove_stream(network::stream* stream);

private:

  /**
   * Don't allow copying of objects.
   */
  service(const service&);

  /**
   * Don't allow assignment of objects.
   */
  service& operator=(const service&);

  network::ip::tcp::connection_service _connection_service;
  network::ip::tcp::listener_service _listener_service;
  client_service _client_service;
};

} /* namespace proxy */
} /* namespace scalestack */

#endif /* SCALESTACK_PROXY_SERVICE_H */
