#!/bin/sh
#
# Scale Stack
#
# Copyright 2010 Eric Day
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Get filename we want to run without path
name=`echo $1 | sed 's/.*\/\(scalestack\/.*[^\/]*\)$/\1/'`

ext=`echo $name | sed 's/.*\.\([^.]*$\)/\1/'`
if [ "x$ext" = "x$name" ]
then
  ext=""
fi

if [ ! "x$ext" = "xsh" ]
then
  libtool_prefix="libtool --mode=execute"
fi

# Only run tests in given list if needed
if [ -n "$SCALESTACK_TEST_ONLY" ]
then
  run="no"
  for x in $SCALESTACK_TEST_ONLY
  do
    if [ "x$x" = "x$name" ]
    then
      run="yes"
      break
    fi
  done

  if [ "x$run" = "xno" ]
  then
    echo "SKIP: $name"
    exit 0
  fi
fi

# Set prefix if it was given through environment
if [ -n "$SCALESTACK_TEST_PREFIX" ]
then
  if [ -n "$SCALESTACK_TEST_FILTER" ]
  then
    # If filter variable is set, only apply prefix to those that match
    for x in $SCALESTACK_TEST_FILTER
    do
      if [ "x$x" = "x$name" ]
      then
        prefix="$libtool_prefix $SCALESTACK_TEST_PREFIX"
        with=" (with prefix after filter)"
        break
      fi
    done
  else
    prefix="$libtool_prefix $SCALESTACK_TEST_PREFIX"
    with=" (with prefix)"
  fi
fi

# Set this to fix broken libtool test
ECHO=`which echo`
export ECHO

# This needs to be set because of broken libtool on OSX
DYLD_LIBRARY_PATH=scalestack/.libs
export DYLD_LIBRARY_PATH

$prefix $1 $SCALESTACK_TEST_ARGS
